from __future__ import print_function
import numpy
from models import MarginDoubleInt as DoubleInt
import pyparticleest.simulator as simulator
import sys
import math

def generate_data(T, Q, R, p0, v0):
    A = numpy.asarray(((1.0, 1.0), (0.0, 1.0)))
    x = numpy.zeros((2, T + 1))
    x[0, 0] = p0
    x[1, 0] = v0
    y = numpy.zeros((T + 1, 1))
    y[0] = x[0, 0] + numpy.random.normal(0.0, math.sqrt(R))
    for i in xrange(T):
        x[:, i + 1] = A.dot(x[:, i])
        x[1, i + 1] += numpy.random.normal(0.0, math.sqrt(Q))

        y[i + 1] = x[0, i + 1] + numpy.random.normal(0.0, math.sqrt(R))

    return (x.T, y)

if __name__ == '__main__':

    num = 50
    nums = 10

    seed_offset = 0
    sims = 10000

    Q = 1.0
    R = 1.0
    T = 50

    p0 = -10
    v0 = 1

    Rvec = numpy.asarray([5, 10, 15, 20, 25, 50, 75, 100, 150, 200, 250, 300, 400, 500])

    print("ID  full_p full_v", end='')
    for r in Rvec:
        print(" R%03d_p R%03d_v" % (r, r), end='')
    print('')
    model = DoubleInt(Q, R, p0, v0)
    for k in range(sims):


        # Create reference
        numpy.random.seed(seed_offset + k)
        (x, y) = generate_data(T, Q, R, p0, v0)

        print("%03d" % (seed_offset + k), end='')

        # seed so that all smoothers start with the same filtered estimate
        numpy.random.seed(seed_offset + k)
        sim = simulator.Simulator(model=model, u=None, y=y)
        sim.simulate(num, nums, res=0.67, filter='PF', smoother='full', meas_first=True)

        smean = sim.get_smoothed_mean()
        err = smean[:-1, (1, 0)] - x[:-1]

        print(" %6.04f %6.04f" % (numpy.sqrt(numpy.mean(err[:, 1] ** 2)),
                                  numpy.sqrt(numpy.mean(err[:, 0] ** 2))), end='')

        # Overwrite previous smoothed estimate
        sim.straj = sim.pt.perform_smoothing(nums, "ancestor")

        for r in xrange(1, Rvec[-1] + 1):
            sim.straj.traj = sim.model.pre_mhips_pass(sim.straj)
            sim.straj.traj = sim.straj.perform_mhips_pass(None, reduced=True)
            sim.straj.traj = sim.model.post_smoothing(sim.straj)

            if ((Rvec == r).any()):
                smean = sim.get_smoothed_mean()
                err = smean[:-1, (1, 0)] - x[:-1]

                print(" %6.04f %6.04f" % (numpy.sqrt(numpy.mean(err[:, 1] ** 2)),
                                          numpy.sqrt(numpy.mean(err[:, 0] ** 2))), end='')

        print('')
        sys.stdout.flush()
